package xyz.lychee.lagfixer.libs.adventure.platform;

import java.util.UUID;
import java.util.function.Function;
import xyz.lychee.lagfixer.libs.adventure.audience.Audience;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.adventure.pointer.Pointered;
import xyz.lychee.lagfixer.libs.adventure.text.flattener.ComponentFlattener;
import xyz.lychee.lagfixer.libs.adventure.text.renderer.ComponentRenderer;
import xyz.lychee.lagfixer.libs.annotations.NotNull;

public interface AudienceProvider extends AutoCloseable {
   @NotNull Audience all();

   @NotNull Audience console();

   @NotNull Audience players();

   @NotNull Audience player(final @NotNull UUID playerId);

   default @NotNull Audience permission(final @NotNull Key permission) {
      return this.permission(permission.namespace() + '.' + permission.value());
   }

   @NotNull Audience permission(final @NotNull String permission);

   @NotNull Audience world(final @NotNull Key world);

   @NotNull Audience server(final @NotNull String serverName);

   @NotNull ComponentFlattener flattener();

   void close();

   public interface Builder<P extends AudienceProvider, B extends Builder<P, B>> {
      @NotNull
      @NotNull B componentRenderer(final @NotNull ComponentRenderer<Pointered> componentRenderer);

      @NotNull
      @NotNull B partition(final @NotNull Function<Pointered, ?> partitionFunction);

      @NotNull
      default <T> @NotNull B componentRenderer(final @NotNull Function<Pointered, T> partition, final @NotNull ComponentRenderer<T> componentRenderer) {
         return this.partition(partition).componentRenderer(componentRenderer.mapContext(partition));
      }

      @NotNull P build();
   }
}
